using System;
using System.Data.Entity;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;
using Ninject;
using Ninject.Modules;
using TourDataManager.Entities;

namespace TourDataManager {

    
    /// <summary>
    /// Фасад для всего плагина
    /// </summary>
    public class TourDataManagerPlugin : IAuthenticator{

        public static volatile string PersistentPath;

        public static string GetPathInPersistent(string relative){
            return Path.Combine(PersistentPath, relative);
        }

        private FetchEstatesUseCase estatesfetch{ get; set; }
        private FetchTourPreviewsUseCase tourpreviewsfetch{ get; set; }
        private Db database{ get; set; }


        private IKernel Container;
        
        public TourDataManagerPlugin(string persistentPath){
            PersistentPath = persistentPath;
            Container = new StandardKernel(new MyModule());
            
            
            estatesfetch = Container.Get<FetchEstatesUseCase>();
            tourpreviewsfetch = Container.Get<FetchTourPreviewsUseCase>();
            database = Container.Get<Db>();
        }

        public async Task<(bool, string)> LoginAsync(string email, string password){
            var result = await Container.Get<IAuthenticator>().LoginAsync(email, password);
            if(result.Item1) SaveCookie();
            return result;
        }

        private void SaveCookie(){
            var cookiestor = Container.Get<CookieStorage>();
            Debug.Log($"Cookie saved. Current count : {cookiestor.Get().Count}");
            cookiestor.Save();
        }

        public void Logout(){ throw new NotImplementedException(); }

        public async Task<Estate[]> FetchEstatesAsync(){
            var estates = await estatesfetch.FetchEstatesFromServerAsync();
            database.InsertEstates(estates);
            return estates;
        }

        public async Task<Tour[]> FetchTourPreviewsAsync(long estateId){
            var tours = await tourpreviewsfetch.FetchTourFromServerAsync(estateId);
            database.InsertTours(tours);
            return tours;
        }

        public ContentLoadingTask DownloadTourContent(long tourId){
            return Container.Get<FetchTourContentUseCase>().FetchTourContentFromServerAsync(tourId);
        }

        public (bool, string) Login(string email, string password){
            return LoginAsync(email, password).Result;
        }
        
        
        
    }

    public class MyModule : NinjectModule {


        public override void Load(){
            var cookieStorage = new CookieStorage(TourDataManagerPlugin.GetPathInPersistent("cookie.storage"));
            var httpClientHandler = new HttpClientHandler();
            //CookieContainer сам запоминает полученные из ответа куки
            httpClientHandler.CookieContainer = cookieStorage.Get();
            var httpClient = new HttpClient(httpClientHandler);
            
            
            Bind<CookieStorage>().ToConstant(cookieStorage).InSingletonScope();
            Bind<HttpClientHandler>().ToConstant(httpClientHandler).InSingletonScope();
            Bind<HttpClient>().ToConstant(httpClient).InSingletonScope();
            Bind<IAuthenticator>().To<Authenticator>().InSingletonScope();
            Bind<FetchEstatesUseCase>().ToSelf().InSingletonScope();
            Bind<FetchTourPreviewsUseCase>().ToSelf().InSingletonScope();
            Bind<FetchTourContentUseCase>().ToSelf().InSingletonScope();
            Bind<Db>().ToSelf().InSingletonScope();

        }
    }
}